# Try to import from module, else import from the source code
try:
    from latynkatar import Cyr2Lat
except ModuleNotFoundError:
    from src.latynkatar import Cyr2Lat


def test_l():
    assert Cyr2Lat.convert("ЛаЭлЯЛуЛіЛюЛЁлЕлЬ лЛя") == "ŁaElAŁuLiLuLOlEl lLa"


def test_ch():
    assert Cyr2Lat.convert("ХаХу ХЫВАХххххх Хіх") == "ChaChu ChYVAChchchchchch Chich"


def test_sz():
    assert Cyr2Lat.convert_classic("ШашуШышшшшшшш") == "SzaszuSzyszszszszszszsz"


def test_š():
    assert Cyr2Lat.convert("ШашуШышшшшшшш") == "ŠašuŠyššššššš"


def test_cz():
    assert Cyr2Lat.convert_classic("чАЧыЧУ") == "czACzyCzU"


def test_č():
    assert Cyr2Lat.convert("чАЧыЧУ") == "čAČyČU"


def test_ż():
    assert Cyr2Lat.convert_classic("жУрАвІнЫЖэЖЫ") == "żUrAwInYŻeŻY"


def test_ž():
    assert Cyr2Lat.convert("жУрАвІнЫЖэЖЫ") == "žUrAvInYŽeŽY"


def test_w():
    assert Cyr2Lat.convert_classic("войт і Ваявода") == "wojt i Wajawoda"


def test_v():
    assert Cyr2Lat.convert("войт і Ваявода") == "vojt i Vajavoda"


def test_miakkaści():
    assert (
        Cyr2Lat.convert("снег смех поспех святы Валянцін жаданні")
        == "śnieh śmiech pośpiech śviaty Valancin žadańni"
    )
    
def test_biez_miakkaści():
    assert (
        Cyr2Lat.convert("снег смех поспех святы Валянцін жаданні", miakkasc=False)
        == "snieh smiech pospiech sviaty Valancin žadanni"
    )



# Try to import from module, else import from the source code
try:
    from latynkatar import Cyr2Lat
except ModuleNotFoundError:
    from src.latynkatar import Cyr2Lat

from time import monotonic

# Узор узяты з часопіса PAMYŁKA:
# https://github.com/PAMYLKA-ZIN/pamylka-number-3/tree/main/PAMYLKA_ZIN_3_FOR_SHARING
PRYKŁAD_PAMYŁKA = """Вітаем цябе, чытачу!
Гэта трэці нумар PAMYŁKA ZIN!
Мы вельмі цешымся, што да каманды стваральнікаў працягваюць далучацца
новыя навукоўцы і мастакі! І мы будзем радыя кожнаму новаму ўдзельніку!
Сябры, мы рэдакцыяй надумалі запачаткаваць прэмію – «Бізон Гіґс». Таму гэты
нумар мы прысвячаем усім беларускім навукоўцам і хочам анансаваць прэмію,
якая будзе ўвасабляць сабой Беларусь і навуку разам! Гэта ўзнагарода для
беларускіх навукоўцаў і даследнікаў дакладных і прыродазнаўчых навук ад
навукова-папулярнага часопісу Pamyłka Zin.
Больш дэталяў апавядае першы артыкул нумару.
"""
UZOR_PAMYŁKA = """Vitajem ciabie, čytaču!
Heta treci numar PAMYŁKA ZIN!
My vielmi ciešymsia, što da kamandy stvaralnikaŭ praciahvajuć dałučacca
novyja navukoŭcy i mastaki! I my budziem radyja kožnamu novamu ŭdzielniku!
Siabry, my redakcyjaj nadumali zapačatkavać premiju – «Bizon Higs». Tamu hety
numar my pryśviačajem usim biełaruskim navukoŭcam i chočam anansavać premiju,
jakaja budzie ŭvasablać saboj Biełaruś i navuku razam! Heta ŭznaharoda dla
biełaruskich navukoŭcaŭ i daślednikaŭ dakładnych i pryrodaznaŭčych navuk ad
navukova-papularnaha časopisu Pamyłka Zin.
Bolš detalaŭ apaviadaje pieršy artykuł numaru.
"""
UZOR_PAMYŁKA_KLASIČNY = """Witajem ciabie, czytaczu!
Heta treci numar PAMYŁKA ZIN!
My wielmi cieszymsia, szto da kamandy stwaralnikaŭ praciahwajuć dałuczacca
nowyja nawukoŭcy i mastaki! I my budziem radyja kożnamu nowamu ŭdzielniku!
Siabry, my redakcyjaj nadumali zapaczatkawać premiju – «Bizon Higs». Tamu hety
numar my pryświaczajem usim biełaruskim nawukoŭcam i choczam anansawać premiju,
jakaja budzie ŭwasablać saboj Biełaruś i nawuku razam! Heta ŭznaharoda dla
biełaruskich nawukoŭcaŭ i daślednikaŭ dakładnych i pryrodaznaŭczych nawuk ad
nawukowa-papularnaha czasopisu Pamyłka Zin.
Bolsz detalaŭ apawiadaje pierszy artykuł numaru.
"""

PRYKŁAD_BAHDANOVIČ = """Маладыя гады,
Маладыя жаданні!
Ні жуды, ні нуды,
Толькі шчасьце каханьня!
 
Помніш толькі красу,
Мілы тварык дзявочы,
Залатую касу,
Сіняватыя вочы!

Цёмны сад-вінаград,
Цьвет бяленькі вішнёвы, —
І агністы пагляд,
І гарачыя словы!
 
Будзь жа, век малады,
Поўны сьветлымі днямі!
Пралятайце, гады,
Залатымі агнямі!
"""
UZOR_BAHDANOVIČ = """Maładyja hady,
Maładyja žadańni!
Ni žudy, ni nudy,
Tolki ščaście kachańnia!
 
Pomniš tolki krasu,
Miły tvaryk dziavočy,
Załatuju kasu,
Siniavatyja vočy!

Ciomny sad-vinahrad,
Ćviet bialeńki višniovy, —
I ahnisty pahlad,
I haračyja słovy!
 
Budź ža, viek małady,
Poŭny śvietłymi dniami!
Pralatajcie, hady,
Załatymi ahniami!
"""
UZOR_BAHDANOVIČ_KLASICZNY = """Maładyja hady,
Maładyja żadańni!
Ni żudy, ni nudy,
Tolki szczaście kachańnia!
 
Pomnisz tolki krasu,
Miły twaryk dziawoczy,
Załatuju kasu,
Siniawatyja woczy!

Ciomny sad-winahrad,
Ćwiet bialeńki wiszniowy, —
I ahnisty pahlad,
I haraczyja słowy!
 
Budź ża, wiek małady,
Poŭny świetłymi dniami!
Pralatajcie, hady,
Załatymi ahniami!
"""

with open("tests/data/novaja_ziamla.txt", "r") as novy_fail:
    NOVAJA_ZIAMLA = novy_fail.read()


def test_z_pamylki():
    assert Cyr2Lat.convert(PRYKŁAD_PAMYŁKA) == UZOR_PAMYŁKA


def test_z_pamylki_klasiczny():
    assert Cyr2Lat.convert_classic(PRYKŁAD_PAMYŁKA) == UZOR_PAMYŁKA_KLASIČNY


def test_bahdanovicz():
    assert Cyr2Lat.convert(PRYKŁAD_BAHDANOVIČ) == UZOR_BAHDANOVIČ


def test_bahdanovicz_klasiczny():
    assert Cyr2Lat.convert_classic(PRYKŁAD_BAHDANOVIČ) == UZOR_BAHDANOVIČ_KLASICZNY


def test_novaj_ziamloju():
    start = monotonic()
    _ = Cyr2Lat.convert(NOVAJA_ZIAMLA)
    finish = monotonic()

    time_required = finish - start

    print(start, finish, time_required)

    assert time_required < 0.5


def test_novaj_ziamloju_klasicny():
    start = monotonic()
    _ = Cyr2Lat.convert_classic(NOVAJA_ZIAMLA)
    finish = monotonic()

    time_required = finish - start

    print(start, finish, time_required)

    assert time_required < 0.5



"""
This file is part of Łatynkatar.

Łatynkatar is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License v3 (LGPLv3) as published by the 
Free Software Foundation, either version 3 of the License, or (at your option) 
any later version.

Łatynkatar is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Lesser General Public License v3 (LGPLv3) for 
more details. In file LICENSE which should came with a package, or look at it
in the repo, see <https://github.com/measles/latynkatar/blob/main/LICENSE>.

You should have received a copy of the GNU Lesser General Public License v3 
(LGPLv3) along with Łatynkatar. If not, see <https://www.gnu.org/licenses/>. 

:copyright: (c) 2023, 2024 by Andrej Zacharevicz.
"""

from .variables import (
    PRAVILY_KANVERTACYJ,
    HALOSNYJA,
    JOTAWANYJA_LITARY,
    ZYCZNYJA_Z_TRANZITAM,
    MOHUC_PAZNACZACCA_JAK_MIAKKIJA,
    KLASICZNYJA_PRAWILY_KANVERTACYJI,
)


def karvertavac_z_j(current_letter: str, previous_letter: str) -> str:
    if (
        not previous_letter
        or previous_letter.lower() in HALOSNYJA
        or not previous_letter.isalpha()
        or previous_letter == "'"
        or previous_letter.lower() == "ь"
    ) and current_letter.lower() != "і":
        base = "j"
    else:
        base = "i"

    second_letter = JOTAWANYJA_LITARY[current_letter.lower()]
    if (
        current_letter.lower() != "і"
        and previous_letter
        and previous_letter.lower() == "л"
    ):
        base = ""

    converted_letter = base + second_letter

    return converted_letter


def convert(text: str, classic: bool = False, miakkasc: bool = True) -> str:
    converted_text = ""
    text_length = len(text)
    biahuczyja_pravily = (
        KLASICZNYJA_PRAWILY_KANVERTACYJI if classic else PRAVILY_KANVERTACYJ
    )

    for index, current_letter in enumerate(text):
        converted_letter = ""
        if index > 0:
            previous_letter = text[index - 1]
        else:
            previous_letter = None

        if index < text_length - 1:
            next_letter = text[index + 1]
        else:
            next_letter = None

        if index < text_length - 2:
            next_next_letter = text[index + 2]
        else:
            next_next_letter = None

        if current_letter.lower() in biahuczyja_pravily:
            converted_letter = biahuczyja_pravily[current_letter.lower()]
        elif current_letter.lower() in MOHUC_PAZNACZACCA_JAK_MIAKKIJA:
            hard, soft = MOHUC_PAZNACZACCA_JAK_MIAKKIJA[current_letter.lower()]
            if next_letter and next_letter.lower() == "ь":
                converted_letter = soft
            elif (
                next_letter
                and current_letter.lower() == "л"
                and next_letter.lower() in JOTAWANYJA_LITARY.keys()
            ):
                converted_letter = soft
            elif (
                miakkasc
                and (next_letter and next_letter.lower() in ZYCZNYJA_Z_TRANZITAM)
                and (
                    next_next_letter
                    and (
                        next_next_letter.lower() in JOTAWANYJA_LITARY
                        or next_next_letter.lower() == "ь"
                    )
                )
            ):
                if current_letter.lower() != "н" or (
                    current_letter.lower() == "н" == next_letter.lower()
                ):
                    converted_letter = soft
                else:
                    converted_letter = hard
            else:
                converted_letter = hard
        elif current_letter.lower() == "ь" or current_letter.lower() == "'":
            pass
        # Перадаюцца праз i/j
        elif current_letter.lower() in JOTAWANYJA_LITARY:
            converted_letter = karvertavac_z_j(current_letter, previous_letter)
        else:
            converted_letter = current_letter

        converted_letter = (
            converted_letter
            if current_letter.islower()
            else converted_letter.capitalize()
        )
        converted_text += converted_letter

    return converted_text


class Cyr2Lat:
    @classmethod
    def convert(cls, text: str, miakkasc: str = True) -> str:
        return convert(text=text, classic=False, miakkasc=miakkasc)

    @classmethod
    def convert_classic(cls, text: str, miakkasc: str = True) -> str:
        return convert(text=text, classic=True, miakkasc=miakkasc)
